// JavaScript Document
/*使用するイベントハンドラではthisがイベント登録対象になるのでAnimeBalloonであるthisを変数で格納*/
var animeTimer;
var animeBalloon;

AnimeBalloon = function(def, pref, change, capture, remove)
{
	try
	{
		animeBalloon = this;
		
		this.animeData = [];
		this.animeData[0] = {x:-2, y:-1,z:6};
		this.animeData[1] = {x:-1, y:0,z:5};
		this.animeData[2] = {x:0, y:1,z:4};
		this.animeData[3] = {x:1, y:2,z:3};
		this.animeData[4] = {x:2, y:3,z:2};
		this.animeData[5] = {x:3, y:4,z:1};
		this.animeData[6] = {x:4, y:5,z:0};
		this.animeData[7] = {x:5, y:6,z:-1};
		this.animeData[8] = {x:6, y:7,z:-2};
		
		
		this.def = def;
		this.prev = def;
		
		this.min = 0;
		this.max = this.animeData.length-1;
		
		this.speedArray = [0.2, 0.5, 1, 2, 5];
		this.speedIndex = 2;
		
		
		
		this.targetID = "";
		
		//変更時に呼び出す関数
		this.changeFnc = change;
		
		//キャプチャ時に呼び出す関数
		this.captureFnc = capture;
		
		//キャプチャ削除時に呼び出す関数
		this.removeFnc = remove;
		
		this.marginTB = 5;
		this.marginLR = 5;
		
		this.loopMode = 0;
		this.loopFlag = true;
		
		initAnimeBalloon();
	}
	catch(e)
	{
		alert("AnimeBalloon setAnimeBalloon:" + e.toString());
	}
};

function initAnimeBalloon()
{
	try
	{
		//再生
		document.getElementById("playBtn").addEventListener("click", playAnimationHandler);
		
		//ループ
		document.getElementById("loopBtn").addEventListener("click", changeLoopHandler);
		
		//速度
		document.getElementById("speedConfig").addEventListener("click", changeSpeedHandler);
		
		//キャプチャ
		document.getElementById("captureBtn").addEventListener("click", captureDataHandler);
		
		//削除
		document.getElementById("removeBtn").addEventListener("click", removeCaptureHandler);
		
		document.getElementById("firstPointBtn").addEventListener("click", changePointDataHandler);
		document.getElementById("backPointBtn").addEventListener("click", changePointDataHandler);
		document.getElementById("nextPointBtn").addEventListener("click", changePointDataHandler);
		document.getElementById("lastPointBtn").addEventListener("click", changePointDataHandler);
	}
	catch(e)
	{
		alert("initAnimeBalloon:" + e.toString());
	}
}

function changePointDataHandler(e)
{
	if(animeFlag == false)
	{
		var f = false;
		
		switch(e.currentTarget.id)
		{
			case "firstPointBtn":
				if(animeBalloon.prev != animeBalloon.min)
				{
					animeBalloon.prev = animeBalloon.min;
					f = true;
				}
				break;
				
			case "backPointBtn":
				if(animeBalloon.prev > animeBalloon.min)
				{
					animeBalloon.prev--;
					f = true;
				}
				break;
				
			case "nextPointBtn":
				if(animeBalloon.prev < animeBalloon.max)
				{
					animeBalloon.prev++;
					f = true;
				}
				break;
				
			case "lastPointBtn":
				if(animeBalloon.prev != animeBalloon.max)
				{
					animeBalloon.prev = animeBalloon.max;
					f = true;
				}
				break;
		}
		
		if(f)
		{
			animeBalloon.changeFnc();
		}
	}
}

AnimeBalloon.prototype.openAnimeBalloon = function(id)
{
	try
	{
		if(this.targetID != id)
		{
			this.targetID = id;
			
			var eX = $("#"+id).position().left;
			var pX = $("#"+id).parent().position().left;
			
			var eW = $("#"+id).outerWidth();
			
			$("#animeTriangle").css("margin-left", eX+pX+eW/2+7);
			
			document.getElementById("animeBalloon").style.display = "inline-block";
		}
		else
		{
			animeBalloon.closeAnimeBalloon();
		}
	}
	catch(e)
	{
		alert("openAnimeBalloon:" + e.toString());
	}
}

AnimeBalloon.prototype.closeAnimeBalloon = function()
{
	try
	{
		this.targetID = "";
			
		document.getElementById("animeBalloon").style.display = "none";
	}
	catch(e)
	{
		alert("closeAnimeBalloon:" + e.toString());
	}
}

function playAnimationHandler(e)
{
	try
	{
		if(animeFlag == false)
		{
			document.getElementById("playBtn").src = "./img/stopIcon.png";

			startAnimation();
		}
		else
		{
			clearTimeout(animeTimer);
			document.getElementById("playBtn").src = "./img/playIcon.png";
		}

		animeFlag = !animeFlag;
	}
	catch(e)
	{
		alert("AnimeBalloon playAnimationHandler:" + e.toString());
	}
}


function startAnimation()
{
	try
	{
		//　1/5秒ごとに表示変更
		var sec = 500 / animeBalloon.speedArray[animeBalloon.speedIndex];
		
		if(animeBalloon.loopMode == 2 && animeBalloon.prev >= animeBalloon.max)
		{
			animeBalloon.prev = animeBalloon.min-1;
		}
		
		animeTimer = setTimeout(stopAnimation, sec);
	}
	catch(e)
	{
		alert("startAnimation:" + e.toString());
	}
}

function stopAnimation()
{
	try
	{
		clearTimeout(animeTimer);
		
		//スライド全体でのデフォルト(1x)でのミリ秒数
		var d = 10000;
		var s = 500 / animeBalloon.speedArray[animeBalloon.speedIndex];
		
		
		if(animeBalloon.loopFlag)
		{
			animeBalloon.prev++;
		}
		else
		{
			animeBalloon.prev--;
		}

		//浮動小数点対策
		animeBalloon.prev = Math.round(animeBalloon.prev * 10000)/10000;
		
		if(animeBalloon.prev < animeBalloon.min)
		{
			animeBalloon.prev = animeBalloon.min;
		}
		else if(animeBalloon.prev > animeBalloon.max)
		{
			animeBalloon.prev = animeBalloon.max;
		}

		animeBalloon.changeFnc();
		
		//逆行込みの繰り返し
		if(animeBalloon.loopMode == 0)
		{
			//順行
			if(animeBalloon.loopFlag == true)
			{
				if(animeBalloon.prev >= animeBalloon.max)
				{
					animeBalloon.loopFlag = false;
				}
			}
			//逆行
			else
			{
				if(animeBalloon.prev <= animeBalloon.min)
				{
					animeBalloon.loopFlag = true;
				}
			}
			
			startAnimation();
		}
		//はじめからの繰り返し
		else if(animeBalloon.loopMode == 1)
		{
			if(animeBalloon.prev >= animeBalloon.max)
			{
				animeBalloon.prev = animeBalloon.min-1;
			}
			
			startAnimation();
		}
		//最後まで行くと停止
		else
		{
			if(animeBalloon.prev < animeBalloon.max)
			{
				startAnimation();
			}
			else
			{
				//アニメーション終了
				document.getElementById("playBtn").src = "./img/playIcon.png";
				animeFlag = false;
			}
		}
	}
	catch(e)
	{
		alert("AnimeBalloon stopAnimation:" + e.toString());
	}
}

function changeLoopHandler(e)
{
	try
	{
		switch(animeBalloon.loopMode)
		{
			case 0:
				animeBalloon.loopMode = 1;
				animeBalloon.loopFlag = true;
				document.getElementById("loopBtn").src = "./img/autoIcon.png";
				break;

			case 1:
				animeBalloon.loopMode = 2;
				animeBalloon.loopFlag = true;
				document.getElementById("loopBtn").src = "./img/normalIcon.png";
				break;

			case 2:
				animeBalloon.loopMode = 0;
				document.getElementById("loopBtn").src = "./img/loopIcon.png";
				break;
		}
	}
	catch(e)
	{
		alert("AnimeBalloon changeLoopHandler:" + e.toString());
	}
}

function changeSpeedHandler(e)
{
	try
	{
		if(animeBalloon.speedIndex+1 < animeBalloon.speedArray.length)
		{
			animeBalloon.speedIndex+=1;
		}
		else
		{
			animeBalloon.speedIndex = 0;
		}
		
		document.getElementById("speedConfig").src = "./img/s"+animeBalloon.speedArray[animeBalloon.speedIndex]+".png";
	}
	catch(e)
	{
		alert("changeSpeedHandler:" + e.toString());
	}
}

function captureDataHandler(e)
{
	try
	{
		animeBalloon.captureFnc();
	}
	catch(e)
	{
		alert(e.toString());
	}
}

function removeCaptureHandler(e)
{
	try
	{
		animeBalloon.removeFnc();
	}
	catch(e)
	{
		alert(e.toString());
	}
}


