// JavaScript Document
function createTextMesh(str, s, tm)
{
	try
	{
		var tg = new THREE.TextGeometry(str,
			{
				size:s, 
				height:0.1,
				curveSegments:10,
				font: "helvetiker",
				weight: "normal",
				style: "normal",
				bevelThickness:1,
				bevelSize:0,
				bevelEnabled: false
			});
		
		var mesh = new THREE.Mesh(tg, tm);
		
		return mesh;
	}
	catch(e)
	{
		alert("createTextMesh:" + e.toString());
	}
}

//辺の見かけ上の選択用のシリンダー
function createLineCylinder(p0, p1, w, m)
{
	try
	{
		var cont = new THREE.Object3D();
		
		changeLineCylinder(cont, p0, p1, w, m);
		
		return cont;
	}
	catch(e)
	{
		alert("createLineCylinder:" + e.toString());
	}
}

function createPlaneObject(w, h, pm, lm)
{
	try
	{
		var obj = new THREE.Object3D();
		var pg = new THREE.PlaneGeometry(w, h, 1, 1);
		var mesh = new THREE.Mesh(pg, pm);
		mesh.name = "planeMesh";
		
		obj.add(mesh);
		
		var p0 = {x:-w/2, y:-h/2, z:0};
		var p1 = {x:w/2, y:-h/2, z:0};
		var p2 = {x:w/2, y:h/2, z:0};
		var p3 = {x:-w/2, y:h/2, z:0};
		
		var line0 = createLineCylinder(p0, p1, 0.1, lm);
		var line1 = createLineCylinder(p0, p3, 0.1, lm);
		var line2 = createLineCylinder(p1, p2, 0.1, lm);
		var line3 = createLineCylinder(p2, p3, 0.1, lm);
		
		obj.add(line0);
		obj.add(line1);
		obj.add(line2);
		obj.add(line3);
		
		return obj;
	}
	catch(e)
	{
		alert("createPlaneObject:" + e.toString());
	}
}

//辺の変更
function changeLineCylinder(cont, p0, p1, w, m)
{
	try
	{
		removeObjectChildren(cont);
		
		var r = getPointRotation(p0, p1);
		var h = getPointLength(p0, p1);
		
		var mesh = new THREE.Mesh(new THREE.CylinderGeometry(w, w, h, 5, 5, false), m);
		
		mesh.position.x = 0;
		mesh.position.y = -h/2;
		mesh.position.z = 0;
		
		cont.add(mesh);
		
		cont.position.x = p0.x;
		cont.position.y = p0.y;
		cont.position.z = p0.z;
		
		cont.rotation.y = r[0];
		cont.rotation.z = r[1]+Math.PI/2;
	}
	catch(e)
	{
		alert("changeCylinderLine:" + e.toString());
	}
}

//矢印の作成
function createArrowObject(p0, p1, w0, w1,　leng, m)
{
	try
	{
		var cont = new THREE.Object3D();
		
		changeArrowObject(cont, p0, p1, w0, w1,　leng, m);
		
		return cont;
	}
	catch(e)
	{
		alert("createArrowObject:" + e.toString());
	}
}

//矢印の変更
function changeArrowObject(cont, p0, p1, w0, w1,　th, m)
{
	try
	{
		removeObjectChildren(cont);
		
		var r = getPointRotation(p0, p1);
		var h = getPointLength(p0, p1);
		
		if(h == 0)
		{
			return;
		}
		
		var lineMesh = new THREE.Mesh(new THREE.CylinderGeometry(w0, w0, h-th, 5, 5, false), m);
		
		var topMesh = new THREE.Mesh(new THREE.CylinderGeometry(w1, 0, th, 5, 5, false), m);
		
		lineMesh.position.y = -(h-th)/2;
		topMesh.position.y = -h+th/2;
		
		cont.add(lineMesh);
		cont.add(topMesh);
		
		var r = getPointRotation(p0, p1);
		
		cont.rotation.y = r[0];
		cont.rotation.z = r[1]+Math.PI/2;
	}
	catch(e)
	{
		alert("changeArrowObject:" + e.toString());
	}
}

function removeObjectChildren(cont)
{
	try
	{
		//子オブジェクト取得
		var children = cont.children;
		
		//子オブジェクト削除
		while(children.length > 0)
		{
			cont.remove(children[0]);
			children.shift();
		}
	}
	catch(e)
	{
		alert("removeObjectChildren:" + e.toString());
	}
}

function axisObject(lengX, lengY, lengZ, w0, w1, leng, lm, tm)
{
	try
	{
		var obj = new THREE.Object3D();
		obj.name = "axisObj";
		
		var span = 5;
		
		//矢印
		var aX = createArrowObject({x:0,y:0,z:-lengX}, {x:0,y:0,z:lengX}, w0, w1, leng, lm);
		var aY = createArrowObject({x:-lengY,y:0,z:0}, {x:lengY,y:0,z:0}, w0, w1, leng, lm);
		var aZ = createArrowObject({x:0,y:-lengZ,z:0}, {x:0,y:lengZ,z:0}, w0, w1, leng, lm);
		
		aX.position.z = -lengX;
		aY.position.x = -lengY;
		aZ.position.y = -lengZ;
		
		obj.add(aX);
		obj.add(aY);
		obj.add(aZ);
		
		var tX = createTextMesh("x", 2, tm);
		tX.name = "textX";
		var tY = createTextMesh("y", 2, tm);
		tY.name = "textY";
		var tZ = createTextMesh("z", 2, tm);
		tZ.name = "textZ";
		var tO = createTextMesh("O", 2, tm);
		tO.name = "textO";
		
		var tSpan = 2;
		
		//テキスト
		tX.position.y = tSpan;
		tX.position.z = lengX;
		
		tY.position.x = lengY-tSpan;
		tY.position.y = tSpan;
		
		tZ.position.x = -tSpan;
		tZ.position.y = lengZ-tSpan;
		
		tO.position.x = 0;
		tO.position.y = -tSpan* 3/2;
		tO.position.z = tSpan/2;
		
		obj.add(tX);
		obj.add(tY);
		obj.add(tZ);
		obj.add(tO);
		
		var i;
		var j=lengX;
		var p0 = {x:0, y:0, z:0};
		var p1 = {x:0, y:0, z:0};
		
		var lineMaterial = new THREE.MeshBasicMaterial({color:"#C1E9FA", opacity:1, transparent:false});
		
		for(i=-j; i<=j; i+=span)
		{
			p0.x = i;
			p0.y = 0;
			p0.z = -lengY;
			
			p1.x = i;
			p1.y = 0;
			p1.z = lengY;
			
			var lineX = createLineCylinder(p0, p1, 0.1, lineMaterial);
			obj.add(lineX);
		}
		
		j=lengY;
		for(i=-j; i<=j; i+=span)
		{
			p0.x = -lengX;
			p0.y = 0;
			p0.z = i;
			
			p1.x = lengX;
			p1.y = 0;
			p1.z = i;
			
			var lineY = createLineCylinder(p0, p1, 0.1, lineMaterial);
			obj.add(lineY);
		}
		
		return obj;
	}
	catch(e)
	{
		alert("axisObject:" + e.toString());
	}
}

//点p0から見た点p1の回転角
function getPointRotation(p0, p1)
{
	try
	{
		var r = new Array();
		
		var d = Math.sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.z-p0.z)*(p1.z-p0.z));
		
		//ｙ軸に合わせた回転
		r[0] = Math.atan2(p0.z-p1.z, p1.x-p0.x);
		//z軸に合わせた回転
		r[1] = Math.atan2(p1.y-p0.y, d);
		
		return r;
	}
	catch(e)
	{
		alert("getPointRotation:" + e.toString());
	}
}
//２点間の距離
function getPointLength(p0, p1)
{
	try
	{
		var length = Math.sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y)+(p1.z-p0.z)*(p1.z-p0.z));
		
		return length;
	}
	catch(e)
	{
		alert("getPointLength:" + e.toString());
	}
}